//---------------------------------------------------------------------------
//
// TScreenSize v1.0
//
// FREEWARE COMPONENT.
//---------------------------------------------------------------------------
// The given component is intended for chanhing screen mode at run time
// Automaticaly restore to origin mode when program being destroying.
// Set Width, Height and BitCount to your mode (ex. 640,480,16)
// and execute Switch procedure to change
// to restore orgigin mode just execute RestoreMode.
//---------------------------------------------------------------------------
//
//  Copyright 1999 Ray Adams
//
//                     IMPORTANT NOTE:
// This software is provided 'as-is', without any express or
// implied warranty. In no event will the author be held
// liable for any damages arising from the use of this
// software.
// Permission is granted to anyone to use this software for
// any purpose, including commercial applications, and to
// alter it and redistribute it freely, subject to the
// following restrictions:
// 1. The origin of this software must not be misrepresented,
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is
//    not required.
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original
//    software.
// 3. This notice may not be removed or altered from any
//    source distribution.
unit ScreenSize;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,DsgnIntf;
type

  TErrorSwitchNotifyEvent = procedure (Sender: TObject;ErrorCode:word) of object;

  TScreenSize = class(TComponent)
  private
    FOnBeforeSwitch: TNotifyEvent;
    FBitsCount: byte;
    FWidth: word;
    FHeight: word;
    FAutoSwitch: boolean;
    FOnErrorSwitch: TErrorSwitchNotifyEvent;
    procedure SetBitsCount(const Value: byte);
    procedure SetHeight(const Value: word);
    procedure SetWidth(const Value: word);
    procedure SetAutoSwitch(const Value: boolean);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create( anOwner :TComponent ); override;
    destructor Destroy; override;
    procedure Switch;
    procedure RestoreMode;
  published
    { Published declarations }
    property OnBeforeSwitch:TNotifyEvent read FOnBeforeSwitch write FOnBeforeSwitch;
    property OnErrorSwitch:TErrorSwitchNotifyEvent read FOnErrorSwitch write FOnErrorSwitch;
    property Width:word read FWidth write SetWidth;
    property Height:word read FHeight write SetHeight;
    property BitsCount:byte read FBitsCount write SetBitsCount;
    property AutoSwitch:boolean read FAutoSwitch write SetAutoSwitch;
  end;

function ChangeDisplaySettingsX(lpDevMode: Pointer; dwFlags: DWORD): Longint; stdcall;
procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ray Adams', [TScreenSize]);
end;

function ChangeDisplaySettingsX; external user32 name 'ChangeDisplaySettingsA';

{ TScreenSize }

constructor TScreenSize.Create(anOwner: TComponent);
begin
inherited Create(anOwner);
Width:=640;
Height:=480;
BitsCount:=16;
end;

destructor TScreenSize.Destroy;
begin
ChangeDisplaySettingsx(nil,0);
inherited Destroy;
end;

procedure TScreenSize.RestoreMode;
begin
ChangeDisplaySettingsx(nil,0);
end;

procedure TScreenSize.SetAutoSwitch(const Value: boolean);
begin
  FAutoSwitch := Value;
end;

procedure TScreenSize.SetBitsCount(const Value: byte);
begin
  FBitsCount := Value;
end;

procedure TScreenSize.SetHeight(const Value: word);
begin
  FHeight := Value;
end;

procedure TScreenSize.SetWidth(const Value: word);
begin
  FWidth := Value;
end;

procedure TScreenSize.Switch;
var
    dMode:TDeviceMode;
    res:word;
begin
dMode.dmPelsWidth:=FWidth;
dMode.dmPelsHeight:=FHeight;
dMode.dmBitsPerPel:=FBitsCount;
dMode.dmDisplayFlags:=DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT;
dMode.dmSize:=SizeOf(dMode);
res:=ChangeDisplaySettings(dmode,0);
if res<>DISP_CHANGE_SUCCESSFUL then if assigned(FOnErrorSwitch) then FOnErrorSwitch(Self,res);
end;

end.
